/* boundary.c */

#include "boundary.h"
#include "flag.h"

#include <assert.h>
#include <math.h>
#include <string.h>

#include "model_compat.h"

#define Case break;case
#define Default break;default

/* Randwerte fuer die Felder U und V werden
 * gemaess den Formeln (14) und (15) gesetzt.
 *
 * u   = 0, u      = 0, j = 1,...,jmax
 *  0,j      imax,j
 *
 * v   = 0, v      = 0, i = 1,...,imax               (14)
 *  i,0      i,jmax
 *
 * v   = -v   , v        = -v      , j = 1,...,jmax
 *  0,j    1,j   imax+1,j    imax,j
 *
 * u   = -u   , u        = -u      , i = 1,...,imax
 *  i,0    i,1   i,jmax+1    i,jmax
 *
 *                                                   (15)
 */
void SETBCOND
( 
    REAL **U, REAL **V, 
    int imax, int jmax,
    int wl, int wr, int wt, int wb, 
    int **FLAG
)
{
    int i,j;
    
    switch( wl )
    {
      case 1: /* Haftbedingung */
        for( j = 1; j <= jmax; j++ )
	{
            U[0][j]      = 0.0;           /* (14) */
	    V[0][j]      = - V[1][j];     /* (15) */	    
	}
      Case 2: /* Rutschbedingung */
        for( j = 1; j <= jmax; j++ )
	{
            U[0][j]      = 0.0;           /*  (1) */
	    V[0][j]      = V[1][j];       /*  (2) */	    
	}
      Case 3: /* Ausstroembedingung */
        for( j = 1; j <= jmax; j++ )
	{
            U[0][j]      = U[1][j];       /*  (3) */
	    V[0][j]      = V[1][j];       /*  (3) */	    
	}
    }
    switch( wr )
    {
      case 1: /* Haftbedingung */
        for( j = 1; j <= jmax; j++ )
	{
	    U[imax][j]   = 0.0;           /* (14) */
   	    V[imax+1][j] = - V[imax][j];  /* (15) */
	}
      Case 2: /* Rutschbedingung */
        for( j = 1; j <= jmax; j++ )
	{
	    U[imax][j]   = 0.0;           /*  (1) */
   	    V[imax+1][j] = V[imax][j];    /*  (2) */
	}
      Case 3: /* Ausstroembedingung */
        for( j = 1; j <= jmax; j++ )
	{
	    U[imax][j]   = U[imax-1][j];  /*  (3) */
   	    V[imax+1][j] = V[imax][j];    /*  (3) */
	}
    }
    switch( wt )
    {
      case 1: /* Haftbedingung */
        for( i = 1; i <= imax; i++ )
	{
	    V[i][jmax]   = 0.0;           /* (14) */
	    U[i][jmax+1] = - U[i][jmax];  /* (15) */
	}
      Case 2: /* Rutschbedingung */
        for( i = 1; i <= imax; i++ )
	{
	    V[i][jmax]   = 0.0;           /*  (1) */
	    U[i][jmax+1] = U[i][jmax];    /*  (2) */
	}
      Case 3: /* Ausstroembedingung */
        for( i = 1; i <= imax; i++ )
	{
	    V[i][jmax]   = V[i][jmax-1];  /*  (3) */
	    U[i][jmax+1] = U[i][jmax];    /*  (3) */
	}
    }
    switch( wb )
    {
      case 1: /* Haftbedingung */
        for( i = 1; i <= imax; i++ )
	{
            V[i][0]      = 0.0;           /* (14) */
	    U[i][0]      = - U[i][1];     /* (15) */
	}
      Case 2: /* Rutschbedingung */
        for( i = 1; i <= imax; i++ )
	{
            V[i][0]      = 0.0;           /*  (1) */
	    U[i][0]      = U[i][1];       /*  (2) */
	}
      Case 3: /* Ausstroembedingung */
        for( i = 1; i <= imax; i++ )
	{
            V[i][0]      = V[i][1];       /*  (3) */
	    U[i][0]      = U[i][1];       /*  (3) */
	}
    }
    for( i = 1; i <= imax; i++ )
    {
	set_progres( (float)i / (float) imax );
        for( j = 1; j <= jmax; j++ )
	    switch( BOUNDARY_FLAGS )
	    {
	      case B_O:
	        U[i][j] = 0.0;
		V[i][j-1] = - V[i+1][j-1];
		V[i][j] = - V[i+1][j];
	      Case B_W:
	        U[i-1][j] = 0.0;
		V[i][j-1] = - V[i-1][j-1];
		V[i][j] = - V[i-1][j];
	      Case B_S:
	        V[i][j-1] = 0.0;
		U[i-1][j] = - U[i-1][j-1];
		U[i][j] = - U[i][j-1];
	      Case B_N:
	        V[i][j] = 0.0;
		U[i-1][j] = - U[i-1][j+1];
		U[i][j] = - U[i][j+1];
	      Case B_NO:
	        U[i][j] = 0.0;
		U[i-1][j] = - U[i-1][j+1];
		V[i][j] = 0.0;
		V[i][j-1] = - V[i+1][j-1];
	      Case B_SO:
	        U[i][j] = 0.0;
		U[i-1][j] = - U[i-1][j-1];
		V[i][j-1] = 0.0;
		V[i][j] = - V[i+1][j];
	      Case B_SW:
	        U[i-1][j] = 0.0;
		U[i][j] = - U[i][j-1];
		V[i][j-1] = 0.0;
		V[i][j] = - V[i-1][j];
	      Case B_NW:
	        U[i-1][j] = 0.0;
		U[i][j] = - U[i][j+1];
		V[i][j] = 0.0;
		V[i][j-1] = - V[i-1][j-1];
	      Default:
	        assert( !VERBOT(FLAG[i][j]) );
	    }
    }
}

void SETSPECBCOND
(
    REAL **U, REAL **V, 
    int imax, int jmax, 
    char *problem, REAL UI, REAL VI
)
{
    if( strcasecmp( "DCavity", problem ) == 0 )
    {
	int i;
	for( i = 1; i <= imax; i++ )
	    U[i][jmax+1] = 2.0 - U[i][jmax];  
    }
    else if( strcasecmp( "Karman", problem ) == 0 )
    {
	int j;
	for( j = 1; j <= jmax; j++ )
	   U[0][j] = UI;
    }
    else if( strcasecmp( "Stufe", problem ) == 0 )
    {
	int j;
	for( j = jmax/2; j <= jmax; j++ )
	   U[0][j] = UI;
    }
}
