/* main.c */

#include "flag.h"
#include "init2.h"
#include "boundary.h"
#include "surface.h"
#include "uvp.h"
#include "visual.h"

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "model_compat.h"

#define SQR(x) ((x)*(x))
#define min(x,y) ((x)<(y)?(x):(y))
#define max(x,y) ((x)<(y)?(y):(x))
#define abs(x) ((x)<0?-(x):(x))

static char Particlefile[] = "particles.bin";
static char Particlefile2[] = "particles.bin";

char *compatibility_string[] = {
    "", 
    "DCavity", 
    "Stufe", 
    "Karman", 
    "Damm", 
    "Tropfen"
};

char *compatibility_file[] = {
    "problem/backward_compatibility/leer", 
    "problem/backward_compatibility/dcavity", 
    "problem/backward_compatibility/stufe", 
    "problem/backward_compatibility/karman", 
    "problem/backward_compatibility/damm", 
    "problem/backward_compatibility/tropfen"
};

extern int choice;
void free_if_necessary( struct DataPackage *pkg );

/* Nach Formel (19) wird die Schrittweite delt
 * fuer den naechsten Zeitschritt berechnet.
 * ist tau negativ, so soll die in READ_PARAMETER
 * eingelesene Zeitschrittweite verwendet werden.
 * 
 *
 *
 *                 Re     1       1    -1   delx    dely
 * delt = tau min( -- ( ----- + ----- )  , ------, ------ )    (19)
 *                  2       2       2      |u   |  |v   |
 *                      delx    dely         max     max
 */
void COMP_delt
( 
    REAL *delt, 
    int imax, int jmax, 
    REAL delx, REAL dely, 
    REAL **U, REAL **V, 
    REAL Re, REAL tau 
)
{
    REAL umax = 0, vmax = 0; /* Betragsmaxima von U und V */
    int i,j;
 
    if( tau < 0 )
        return;

    assert( delx != 0 );
    assert( dely != 0 );
    assert( delt != 0 );
    for( i = 1; i <= imax; i++ )
        for( j = 1; j <= jmax; j++ )
        {
            umax = max( umax, abs( U[i][j] ) );    
            vmax = max( vmax, abs( V[i][j] ) );    
        }  
    *delt = Re / (2 * ( 1 / SQR(delx) + 1 / SQR(dely) ));
    if( umax > 0.0 )
        *delt = min( *delt, delx / umax );
    if( vmax > 0.0 )
        *delt = min( *delt, dely / vmax );
    *delt *= tau;
}

/* Ausdruck der momentanen Geometrie */
#ifndef NDEBUG
#define PGEOM print_geometry( model.imax, model.jmax, model.FLAG )
#else
#define PGEOM
#endif
void print_geometry
(
    int imax, int jmax, 
    int **FLAG
)
{
    int i, j;
    printf( "Geometrie:\n" );
    printf( "/" );
    for( i = 1; i <= imax; i++ )
        printf( "-" );
    printf( "\\\n" );
    for( j = jmax; j >= 1; j-- )
    {
        printf( "|" );
        for( i = 1; i <= imax; i++ )
	{
	    int f = FLAG[i][j];
	    char c;
	    if( HINDER(f) )
	        if( VERBOT(f) )
		    c = 'v';
	        else if( ECK(f) )
		    c = 'e';
	        else if( KANTE(f) )
		    c = 'k';
	        else
	            c = '*';
	    else if( LEER(f) )
	        c = ' ';
	    else if( OBERFL(f) )
	        c = 'o';
	    else if( INNER(f) )
	        c = 'O';
	    else 
	        c = 'V';
	    printf( "%c", c );
	}
	printf( "|\n" );
    }
    printf( "\\" );
    for( i = 1; i <= imax; i++ )
        printf( "-" );
    printf( "/\n" );
}

/* Zusammenfassung des Algorithmus s. Seite 13 */

static struct 
{
    char filename[50];
    char problem[50];
    int wl, wr, wt, wb;
    char visualisierung[50];
    int N, N2;
    REAL x1, y1, x2, y2;
    REAL delt_trace, delt_inject, delt_streak;
    int ppc;
    int n;
    REAL last_t;
    REAL t_end;
    REAL del_vec; 
    REAL UI, VI, PI;
    struct particleline *Particlelines, *Particlelines2;
} compatibility_parms;

void init_compatible()
{
    int success = 0;
    free_if_necessary( &model );
    strcpy( compatibility_parms.filename, compatibility_file[choice] );
    compatibility_parms.last_t = 0.0;
    compatibility_parms.Particlelines = 0;
    compatibility_parms.Particlelines2 = 0;
    compatibility_parms.N = compatibility_parms.N2 = 0;
   /* Einlesen der Problemparameter */
    success = READ_PARAMETER( compatibility_parms.filename,
        &model.parms.xlength, &model.parms.ylength, 
	&model.imax, &model.jmax,
        &model.parms.delx, &model.parms.dely, &model.delt, 
	&compatibility_parms.t_end, 
        &model.parms.tau, &model.parms.itermax, 
	&model.parms.eps, &model.parms.omg, 
	&model.parms.alpha, &model.parms.Re,
        &model.parms.GX, &model.parms.GY, 
	&compatibility_parms.UI, &compatibility_parms.VI, &compatibility_parms.PI,
        &compatibility_parms.del_vec,
        compatibility_parms.problem, 
	&compatibility_parms.wl, &compatibility_parms.wr, 
	&compatibility_parms.wt, &compatibility_parms.wb, 
	compatibility_parms.visualisierung, 
	&compatibility_parms.N, 
	&compatibility_parms.x1, &compatibility_parms.y1, 
	&compatibility_parms.x2, &compatibility_parms.y2, 
	&compatibility_parms.delt_trace, 
	&compatibility_parms.delt_inject, 
	&compatibility_parms.delt_streak,
	&compatibility_parms.ppc
    );
    assert( success );
    alloc_if_necessary( &model, TRUE, TRUE );
    /* Setze */ model.time = 0; compatibility_parms.n = 0;
    /* Belege u, v, p mit Anfangswerten */
    INIT_UVP
    ( 
	model.U, model.V, model.P, 
	model.imax, model.jmax, 
	compatibility_parms.UI, compatibility_parms.VI, 
	compatibility_parms.PI, compatibility_parms.problem );
    /* Initialisiere Hindernisse */
    INIT_FLAG
    ( 
	compatibility_parms.problem, 
	model.FLAG, 
	model.imax, model.jmax 
    );
    /* Initialisiere Particle-Tacing und Streaklines */
    if( strcasecmp( "Bahnlinien", compatibility_parms.visualisierung ) == 0
    ||  strcasecmp( "Streichlinien", compatibility_parms.visualisierung ) == 0 )
        compatibility_parms.Particlelines = SET_PARTICLES
	( 
	    compatibility_parms.N, 
	    compatibility_parms.x1, compatibility_parms.y1, 
	    compatibility_parms.x2, compatibility_parms.y2 
	);
    /* Initialisierung der freien Randwert Partikel */
    compatibility_parms.Particlelines2 = INIT_PARTICLES 
    (
	& compatibility_parms.N2, model.imax, model.jmax,
	model.parms.delx, model.parms.dely, 
	compatibility_parms.ppc,
	compatibility_parms.problem, 
	model.U, model.V
    );
}

void calculate( struct MasterPackage *pkg )
{
        set_time( model.time );
	/* Markierung des Fluessigkeitsgebietes */
	set_state( CS_MARK );
	if( compatibility_parms.N2 > 0 )
	{
	    MARK_CELLS
	    (
		model.FLAG, model.imax, model.jmax,
		model.parms.delx, model.parms.dely,
		compatibility_parms.N2, compatibility_parms.Particlelines2
	    );
	    SET_UVP_SURFACE
	    (
		model.U, model.V, model.P, model.FLAG,
		model.imax, model.jmax, model.parms.Re,
		model.parms.delx, model.parms.dely, model.delt,
		model.parms.GX, model.parms.GY
	    );
	}
#if 0
	PGEOM;
#endif	
        /* Waehle delt nach (19) */
        COMP_delt
	( 
	    &model.delt, 
	    model.imax, model.jmax, 
	    model.parms.delx, model.parms.dely, 
	    model.U, model.V, 
	    model.parms.Re, model.parms.tau 
	);
	set_state( CS_BCOND );
	/* Setze die Randwerte fuer u und v gemaess (14), (15) */
        SETBCOND
	( 
	    model.U, model.V, 
	    model.imax, model.jmax, 
	    compatibility_parms.wl, compatibility_parms.wr, 
	    compatibility_parms.wt, compatibility_parms.wb, 
	    model.FLAG 
	);
	SETSPECBCOND
	( 
	    model.U, model.V, 
	    model.imax, model.jmax, 
	    compatibility_parms.problem, 
	    compatibility_parms.UI, compatibility_parms.VI 
	);
	set_state( CS_SOR );
	/* Berechne F und G gemaess (9), (10) und (17) */
        COMP_FG
	( 
	    model.U, model.V, 
	    model.F, model.G, 
	    model.imax, model.jmax, 
	    model.delt, 
	    model.parms.delx, model.parms.dely, 
	    model.parms.GX, model.parms.GY, 
	    model.parms.alpha, model.parms.Re, 
	    model.FLAG 
	);
	/* Berechne die rechte Seite der Druckgleichung (11) */
        COMP_RHS
	( 
	    model.F, model.G, 
	    model.RHS, 
	    model.imax, model.jmax, 
	    model.delt, 
	    model.parms.delx, model.parms.dely 
	);
	/* SOR-Iteration */
	POISSON
	( 
	    model.P, 
	    model.RHS, 
	    model.U, model.V, 
	    model.imax, model.jmax, 
	    model.parms.delx, model.parms.dely, 
	    model.parms.eps, model.parms.itermax, 
	    model.parms.omg, &model.res, 
	    model.FLAG 
	);
	/* Berechne u und v gemaess (7), (8) */
	ADAP_UV
	( 
	    model.U, model.V, 
	    model.F, model.G, 
	    model.P, 
	    model.imax, model.jmax, 
	    model.delt, 
	    model.parms.delx, model.parms.dely, 
	    model.FLAG 
	);
	/* Gegebenenfalls Ausgabe der Werte zur Visualisierung */
	set_residuum( model.res );
	if( strcasecmp( "Bahnlinien", compatibility_parms.visualisierung ) == 0 )
	    PARTICLE_TRACING
	    (
		Particlefile,
		compatibility_parms.delt_trace,
		model.time, model.imax, model.jmax, 
		model.parms.delx, model.parms.dely, model.delt,
		model.U, model.V,
		compatibility_parms.N, compatibility_parms.Particlelines, 
		model.parms.xlength, model.parms.ylength, 
		model.FLAG
	    );
	else if( strcasecmp( "Streichlinien", compatibility_parms.visualisierung ) == 0 )
	    STREAKLINES
	    (
		Particlefile,
		compatibility_parms.delt_inject,
		compatibility_parms.delt_streak,
		model.time, model.imax, model.jmax, 
		model.parms.delx, model.parms.dely, model.delt,
		model.U, model.V,
		compatibility_parms.N, compatibility_parms.Particlelines, 
		model.parms.xlength, model.parms.ylength, 
		model.FLAG
	    );
	/* Bewegung der Partikel beim freien Randwertprobelm */
	if( compatibility_parms.N2 > 0 )
	    ADVANCE_PARTICLES
	    (
		model.imax, model.jmax,
		model.parms.delx, model.parms.dely, model.delt,
		model.U, model.V,
		compatibility_parms.N2, compatibility_parms.Particlelines2,
		model.FLAG
	    );
	INJECT_PARTICLES
	(
	    compatibility_parms.N, 
	    compatibility_parms.Particlelines
	);

	set_time( model.time + model.delt );
}
